@@ -10,6 +10,12 @@ preserved.
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <limits.h>
+#ifndef PATH_MAX
+# ifdef Q_OS_WIN
+#  define PATH_MAX 260
+# endif
+#endif
 
 class MocTestObject : public QObject
 {
@@ -76,7 +82,7 @@ QString qc_findprogram(const QString &prog)
 QString qc_findself(const QString &argv0)
 {
 #ifdef Q_OS_WIN
-	if(argv0.contains('\\\\'))
+	if(argv0.contains('\\'))
 #else
 	if(argv0.contains('/'))
 #endif
@@ -175,12 +181,77 @@ bool qc_removedir(const QString &dirPath)
 	return true;
 }
 
+// simple command line arguemnts splitter able to understand quoted args.
+// the splitter removes quotes and unescapes symbols as well.
+QStringList qc_splitflags(const QString &flags)
+{
+	QStringList ret;
+	bool searchStart = true;
+	bool inQuotes = false;
+	bool escaped = false;
+	QChar quote, backslash = QLatin1Char('\\');
+	QString buf;
+	buf.reserve(PATH_MAX);
+	for (int i=0; i < flags.length(); i++) {
+		if (searchStart && flags[i].isSpace()) {
+			continue;
+		}
+		if (searchStart) {
+			searchStart = false;
+			buf.clear();
+		}
+		if (escaped) {
+			buf += flags[i];
+			escaped = false;
+			continue;
+		}
+		//buf += flags[i];
+		if (inQuotes) {
+			if (quote == QLatin1Char('\'')) {
+				if (flags[i] == quote) {
+					inQuotes = false;
+					continue;
+				}
+			} else { // we are in double quoetes
+				if (flags[i] == backslash && i < flags.length() - 1 &&
+						(flags[i+1] == QLatin1Char('"') || flags[i+1] == backslash))
+				{
+					// if next symbol is one of in parentheses ("\)
+					escaped = true;
+					continue;
+				}
+			}
+		} else {
+			if (flags[i].isSpace()) {
+				ret.append(buf);
+				searchStart = true;
+				buf.clear();
+				continue;
+#ifndef Q_OS_WIN /* on windows backslash is just a path separator */
+			} else if (flags[i] == backslash) {
+				escaped = true;
+				continue; // just add next symbol
+#endif
+			} else if (flags[i] == QLatin1Char('\'') || flags[i] == QLatin1Char('"')) {
+				inQuotes = true;
+				quote = flags[i];
+				continue;
+			}
+		}
+		buf += flags[i];
+	}
+	if (buf.size()) {
+		ret.append(buf);
+	}
+	return ret;
+}
+
 void qc_splitcflags(const QString &cflags, QStringList *incs, QStringList *otherflags)
 {
 	incs->clear();
 	otherflags->clear();
 
-	QStringList cflagsList = cflags.split(" ");
+	QStringList cflagsList = qc_splitflags(cflags);
 	for(int n = 0; n < cflagsList.count(); ++n)
 	{
 		QString str = cflagsList[n];
@@ -209,6 +280,92 @@ QString qc_escapeArg(const QString &str)
 	return out;
 }
 
+
+QString qc_trim_char(const QString &s, const QChar &ch)
+{
+	if (s.startsWith(ch) && s.endsWith(ch)) {
+		return s.mid(1, s.size() - 2);
+	}
+	return s;
+}
+
+// removes surrounding quotes, removes trailing slashes, converts to native separators.
+// accepts unescaped but possible quoted path
+QString qc_normalize_path(const QString &str)
+{
+	QString path = str.trimmed();
+	path = qc_trim_char(path, QLatin1Char('"'));
+	path = qc_trim_char(path, QLatin1Char('\''));
+
+	// It's OK to use unix style'/' pathes on windows Qt handles this without any problems.
+	// Using Windows-style '\\' can leads strange compilation error with MSYS which uses
+	// unix style.
+	QLatin1Char nativeSep('/');
+#ifdef Q_OS_WIN
+	path.replace(QLatin1Char('\\'), QLatin1Char('/'));
+#endif
+	// trim trailing slashes
+	while (path.length() && path[path.length() - 1] == nativeSep) {
+		path.resize(path.length() - 1);
+	}
+	return path;
+}
+
+// escape filesystem path to be added to qmake pro/pri file.
+QString qc_escape_string_var(const QString &str)
+{
+	QString path = str;
+	path.replace(QLatin1Char('\\'), QLatin1String("\\\\"))
+			.replace(QLatin1Char('"'), QLatin1String("\\\""));
+	if (path.indexOf(QLatin1Char(' ')) != -1) { // has spaces
+		return QLatin1Char('"') + path + QLatin1Char('"');
+	}
+	return path;
+}
+
+// escapes each path in incs and join into single string suiable for INCLUDEPATH var
+QString qc_prepare_includepath(const QStringList &incs)
+{
+	if (incs.empty()) {
+		return QString();
+	}
+	QStringList ret;
+	foreach (const QString &path, incs) {
+		ret.append(qc_escape_string_var(path));
+	}
+	return ret.join(QLatin1String(" "));
+}
+
+// escapes each path in libs and to make it suiable for LIBS var
+// notice, entries of libs are every single arg for linker.
+QString qc_prepare_libs(const QStringList &libs)
+{
+	if (libs.isEmpty()) {
+		return QString();
+	}
+	QSet<QString> pathSet;
+	QStringList paths;
+	QStringList ordered;
+	foreach (const QString &arg, libs) {
+		if (arg.startsWith(QLatin1String("-L"))) {
+			QString path = qc_escape_string_var(arg.mid(2));
+			if (!pathSet.contains(path)) {
+				pathSet.insert(path);
+				paths.append(path);
+			}
+		} else if (arg.startsWith(QLatin1String("-l"))) {
+			ordered.append(arg);
+		} else {
+			ordered.append(qc_escape_string_var(arg));
+		}
+	}
+	QString ret;
+	if (paths.size()) {
+		ret += (QLatin1String(" -L") + paths.join(QLatin1String(" -L")) + QLatin1Char(' '));
+	}
+	return ret + ordered.join(QLatin1String(" "));
+}
+
 //----------------------------------------------------------------------------
 // ConfObj
 //----------------------------------------------------------------------------
@@ -375,14 +532,34 @@ QString Conf::qvar(const QString &s)
 	return vars.value(s);
 }
 
+QString Conf::normalizePath(const QString &s) const
+{
+	return qc_normalize_path(s);
+}
+
+QString Conf::escapeQmakeVar(const QString &s) const
+{
+	return qc_escape_string_var(s);
+}
+
+QString Conf::escapedIncludes() const
+{
+	return qc_prepare_includepath(INCLUDEPATH);
+}
+
+QString Conf::escapedLibs() const
+{
+	return qc_prepare_libs(LIBS);
+}
+
 QString Conf::expandIncludes(const QString &inc)
 {
-	return QString("-I") + inc;
+	return QLatin1String("-I") + inc;
 }
 
 QString Conf::expandLibs(const QString &lib)
 {
-	return QString("-L") + lib;
+	return QLatin1String("-L") + lib;
 }
 
 int Conf::doCommand(const QString &s, QByteArray *out)
@@ -396,7 +573,7 @@ int Conf::doCommand(const QString &s, QByteArray *out)
 int Conf::doCommand(const QString &prog, const QStringList &args, QByteArray *out)
 {
 	QString fullcmd = prog;
-	QString argstr = args.join(" ");
+	QString argstr = args.join(QLatin1String(" "));
 	if(!argstr.isEmpty())
 		fullcmd += QString(" ") + argstr;
 	debug(QString("[%1]").arg(fullcmd));
@@ -412,10 +589,14 @@ bool Conf::doCompileAndLink(const QString &filedata, const QStringList &incs, co
 #else
 	QDir tmp(".qconftemp");
 #endif
+	QStringList normalizedLibs;
+	foreach (const QString &l, qc_splitflags(libs)) {
+		normalizedLibs.append(qc_normalize_path(l));
+	}
 
 	if(!tmp.mkdir("atest"))
 	{
-		debug("unable to create atest dir");
+		debug(QString("unable to create atest dir: %1").arg(tmp.absoluteFilePath("atest")));
 		return false;
 	}
 	QDir dir(tmp.filePath("atest"));
@@ -447,11 +628,12 @@ bool Conf::doCompileAndLink(const QString &filedata, const QStringList &incs, co
 		"CONFIG  -= qt app_bundle\n"
 		"DESTDIR  = $$PWD\n"
 		"SOURCES += atest.cpp\n");
-	QString inc = incs.join(" ");
+	QString inc = qc_prepare_includepath(incs);
 	if(!inc.isEmpty())
 		pro += "INCLUDEPATH += " + inc + '\n';
-	if(!libs.isEmpty())
-		pro += "LIBS += " + libs + '\n';
+	QString escaped_libs = qc_prepare_libs(normalizedLibs);
+	if(!escaped_libs.isEmpty())
+		pro += "LIBS += " + escaped_libs + '\n';
 	pro += proextra;
 
 	fname = dir.filePath("atest.pro");
@@ -512,10 +694,7 @@ bool Conf::doCompileAndLink(const QString &filedata, const QStringList &incs, co
 
 bool Conf::checkHeader(const QString &path, const QString &h)
 {
-	QFileInfo fi(path + '/' + h);
-	if(fi.exists())
-		return true;
-	return false;
+	return QDir(path).exists(h);
 }
 
 bool Conf::findHeader(const QString &h, const QStringList &ext, QString *inc)
@@ -652,7 +831,7 @@ bool Conf::findFooConfig(const QString &path, QString *version, QStringList *inc
 	*version = version_out;
 	*incs = incs_out;
 	*libs = libs_out;
-	*otherflags = otherflags_out.join(" ");
+	*otherflags = otherflags_out.join(QLatin1String(" "));
 	return true;
 }
 
@@ -716,7 +895,7 @@ bool Conf::findPkgConfig(const QString &name, VersionMode mode, const QString &r
 	*version = version_out;
 	*incs = incs_out;
 	*libs = libs_out;
-	*otherflags = otherflags_out.join(" ");
+	*otherflags = otherflags_out.join(QLatin1String(" "));
 	return true;
 }
 
@@ -731,19 +910,20 @@ void Conf::addDefine(const QString &str)
 
 void Conf::addLib(const QString &str)
 {
-	if(LIBS.isEmpty())
-		LIBS = str;
-	else
-		LIBS += QString(" ") + str;
+	QStringList libs = qc_splitflags(str);
+	foreach (const QString &lib, libs) {
+		if (lib.startsWith("-l")) {
+			LIBS.append(lib);
+		} else {
+			LIBS.append(qc_normalize_path(lib)); // we don't care about -L prefix since normalier does not touch it.
+		}
+	}
 	debug(QString("LIBS += %1").arg(str));
 }
 
 void Conf::addIncludePath(const QString &str)
 {
-	if(INCLUDEPATH.isEmpty())
-		INCLUDEPATH = str;
-	else
-		INCLUDEPATH += QString(" ") + str;
+	INCLUDEPATH.append(qc_normalize_path(str));
 	debug(QString("INCLUDEPATH += %1").arg(str));
 }
 
@@ -762,8 +942,9 @@ void Conf::addExtra(const QString &str)
 # include"modules.cpp"
 #endif
 
-int main()
+int main(int argc, char ** argv)
 {
+	QCoreApplication app(argc, argv);
 	Conf *conf = new Conf;
 	ConfObj *o = 0;
 	Q_UNUSED(o);
@@ -821,21 +1002,22 @@ int main()
 
 		QString str;
 		str += "# qconf\n\n";
+		str += "greaterThan(QT_MAJOR_VERSION, 4):CONFIG += c++11\n";
 
 		QString var;
-		var = qc_getenv("PREFIX");
+		var = qc_normalize_path(qc_getenv("PREFIX"));
 		if(!var.isEmpty())
 			str += QString("PREFIX = %1\n").arg(var);
-		var = qc_getenv("BINDIR");
+		var = qc_normalize_path(qc_getenv("BINDIR"));
 		if(!var.isEmpty())
 			str += QString("BINDIR = %1\n").arg(var);
-		var = qc_getenv("INCDIR");
+		var = qc_normalize_path(qc_getenv("INCDIR"));
 		if(!var.isEmpty())
 			str += QString("INCDIR = %1\n").arg(var);
-		var = qc_getenv("LIBDIR");
+		var = qc_normalize_path(qc_getenv("LIBDIR"));
 		if(!var.isEmpty())
 			str += QString("LIBDIR = %1\n").arg(var);
-		var = qc_getenv("DATADIR");
+		var = qc_normalize_path(qc_getenv("DATADIR"));
 		if(!var.isEmpty())
 			str += QString("DATADIR = %1\n").arg(var);
 		str += '\n';
@@ -849,13 +1031,17 @@ int main()
 		if(!conf->DEFINES.isEmpty())
 			str += "DEFINES += " + conf->DEFINES + '\n';
 		if(!conf->INCLUDEPATH.isEmpty())
-			str += "INCLUDEPATH += " + conf->INCLUDEPATH + '\n';
+			str += "INCLUDEPATH += " + qc_prepare_includepath(conf->INCLUDEPATH) + '\n';
 		if(!conf->LIBS.isEmpty())
-			str += "LIBS += " + conf->LIBS + '\n';
+			str += "LIBS += " + qc_prepare_libs(conf->LIBS) + '\n';
 		if(!conf->extra.isEmpty())
 			str += conf->extra;
 		str += '\n';
 
+		var = qc_getenv("QC_EXTRACONF");
+		if (!var.isEmpty())
+			str += ("\n# Extra conf from command line\n" + var + "\n");
+
 		QByteArray cs = str.toLatin1();
 		f.write(cs);
 		f.close();
