@@ -26,7 +26,7 @@
 
 #include "stringhelp.h"
 
-#define VERSION "1.5"
+#define VERSION "2.0"
 
 #define CONF_USAGE_SPACE 4
 #define CONF_WRAP        78
@@ -44,7 +44,7 @@ QString escapeFile(const QString &str)
 {
 	QString out;
 	for(int n = 0; n < (int)str.length(); ++n) {
-		if(str[n] == '$' || str[n] == '`')
+		if(str[n] == '$' || str[n] == '`' || str[n] == '\\')
 			out += '\\';
 		out += str[n];
 	}
@@ -150,34 +150,6 @@ static QByteArray embed_file(const QString &name, const QByteArray &data)
 	return out;
 }
 
-// eat double-backslashes on windows, as we don't need the extra escaping to
-//   satisfy the unix shell script
-static QByteArray filter_backslashes(const QByteArray &in)
-{
-	QByteArray out;
-
-	bool inbs = false;
-	for(int n = 0; n < in.size(); ++n)
-	{
-		if(in[n] == '\\')
-		{
-			if(inbs)
-			{
-				inbs = false;
-				continue;
-			}
-
-			inbs = true;
-		}
-		else
-			inbs = false;
-
-		out += in[n];
-	}
-
-	return out;
-}
-
 static QByteArray get_configexe_stub()
 {
 	QFile f;
@@ -194,11 +166,11 @@ static QByteArray get_configexe_stub()
 }
 
 const char *qt4_info_str =
-	"Be sure you have a proper Qt 4.0 build environment set up.  This means not just Qt, "
+	"Be sure you have a proper Qt 4.0+ build environment set up.  This means not just Qt, "
 	"but also a C++ compiler, a make tool, and any other packages necessary "
 	"for compiling C++ programs.\n"
 	"\n"
-	"If you are certain everything is installed, then it could be that Qt 4 is not being "
+	"If you are certain everything is installed, then it could be that Qt is not being "
 	"recognized or that a different version of Qt is being detected by mistake (for example, "
 	"this could happen if \\$QTDIR is pointing to a Qt 3 installation).  At least one of "
 	"the following conditions must be satisfied:\n"
@@ -213,17 +185,18 @@ const char *qt4_info_str =
 	"\n";
 
 const char *qt4_info_str_win =
-	"Be sure you have a proper Qt 4.0 build environment set up.  This means not just Qt, "
+	"Be sure you have a proper Qt 4.0+ build environment set up.  This means not just Qt, "
 	"but also a C++ compiler, a make tool, and any other packages necessary "
 	"for compiling C++ programs.\n"
 	"\n"
-	"If you are certain everything is installed, then it could be that Qt 4 is not being "
+	"If you are certain everything is installed, then it could be that Qt is not being "
 	"recognized or that a different version of Qt is being detected by mistake (for example, "
 	"this could happen if %QTDIR% is pointing to a Qt 3 installation).  At least one of "
 	"the following conditions must be satisfied:\n"
 	"\n"
 	" 1) --qtdir is set to the location of Qt\n"
 	" 2) %QTDIR% is set to the location of Qt\n"
+	" 3) qmake is in the %PATH%\n"
 	"\n"
 	"This program will use the first one it finds to be true, checked in the above order.\n"
 	"\n";
@@ -362,6 +335,8 @@ public:
 		if(libmode)
 			mainopts += ConfOpt("static", QString(), "QC_STATIC", "Create a static library instead of shared.");
 
+		mainopts += ConfOpt("extraconf", "conf", "QC_EXTRACONF", "Extra configuration for nonstandard cases.");
+
 		QString str;
 		str += genHeader();
 		str += genUsage();
@@ -406,7 +381,7 @@ public:
 
 		if(qt4) {
 			if(byoq) {
-				str += "printf \"Preparing internal Qt 4 build environment ... \"\n\n";
+				str += "printf \"Preparing internal Qt 4+ build environment ... \"\n\n";
 
 				str += "cd byoq\n";
 				str += "./byoq build\n";
@@ -425,14 +400,10 @@ public:
 				str += "qm=$PWD/byoq/qt/bin/qmake\n\n";
 			}
 
-			str += "printf \"Verifying Qt 4 build environment ... \"\n\n";
+			str += "printf \"Verifying Qt build environment ... \"\n\n";
 			if(!byoq)
 				str += genQt4Checks();
 		}
-		else {
-			str += "printf \"Verifying Qt 3.x Multithreaded (MT) build environment ... \"\n\n";
-			str += genQt3Checks();
-		}
 
 		str += genEmbeddedFiles();
 
@@ -442,6 +413,7 @@ public:
 			str += QString("export %1\n").arg(i.var);
 		}
 		str += "export QC_VERBOSE\n"; // export verbose flag also
+		str += "export QC_QTSELECT\n";
 
 		str += genDoQConf();
 
@@ -462,9 +434,23 @@ public:
 		mainopts.clear();
 		mainopts += ConfOpt("qtdir", "path", "EX_QTDIR", "Directory where Qt is installed.");
 
+		if(usePrefix) {
+			mainopts += ConfOpt("prefix", "path", "PREFIX", "Base path for build/install.  No default.");
+			if(useBindir)
+				mainopts += ConfOpt("bindir", "path", "BINDIR", "Directory for binaries.  Default: PREFIX/bin");
+			if(useIncdir)
+				mainopts += ConfOpt("includedir", "path", "INCDIR", "Directory for headers.  Default: PREFIX/include");
+			if(useLibdir)
+				mainopts += ConfOpt("libdir", "path", "LIBDIR", "Directory for libraries.  Default: PREFIX/lib");
+			if(useDatadir)
+				mainopts += ConfOpt("datadir", "path", "DATADIR", "Directory for data.  Default: PREFIX/share");
+		}
+
 		if(libmode)
 			mainopts += ConfOpt("static", QString(), "QC_STATIC", "Create a static library instead of shared.");
 
+		mainopts += ConfOpt("extraconf", "conf", "QC_EXTRACONF", "Extra configuration for nonstandard cases");
+
 		// combine main and extra opts together
 		all = mainopts + appopts + depopts;
 
@@ -498,11 +484,11 @@ public:
 
 		write32((quint8 *)buf.data(), 5);
 		out += buf;
-		out += embed_file("modules.cpp", filter_backslashes(filemodulescpp));
-		out += embed_file("modules_new.cpp", filter_backslashes(filemodulesnewcpp));
-		out += embed_file("conf4.h", filter_backslashes(fileconfh));
-		out += embed_file("conf4.cpp", filter_backslashes(fileconfcpp));
-		out += embed_file("conf4.pro", filter_backslashes(fileconfpro));
+		out += embed_file("modules.cpp", filemodulescpp);
+		out += embed_file("modules_new.cpp", filemodulesnewcpp);
+		out += embed_file("conf4.h", fileconfh);
+		out += embed_file("conf4.cpp", fileconfcpp);
+		out += embed_file("conf4.pro", fileconfpro);
 
 		out += lenval(name.toLatin1());
 		out += lenval(profile.toLatin1());
@@ -534,8 +520,6 @@ private:
 		//"else\n"
 		if(qt4)
 			str += "echo \"Good, your configure finished.  Now run $MAKE.\"\n";
-		else
-			str += "echo \"Good, your configure finished.  Now run 'make'.\"\n";
 		//"fi\n"
 		str += "echo\n";
 		return str;
@@ -586,6 +570,7 @@ private:
 
 		QList<ConfUsageOpt> list = optsToUsage(mainopts);
 		list += ConfUsageOpt("verbose",  "",   "Show extra configure output.");
+		list += ConfUsageOpt("qtselect",  "N",   "Select major Qt version (4 or 5).");
 		list += ConfUsageOpt("help",   "",     "This help text.");
 		str += genUsageSection("Main options:", list);
 
@@ -611,6 +596,7 @@ private:
 
 		QList<ConfUsageOpt> list = optsToUsage(mainopts);
 		list += ConfUsageOpt("verbose",  "",   "Show extra configure output.");
+		list += ConfUsageOpt("qtselect",  "N",   "Select major Qt version (4 or 5).");
 		list += ConfUsageOpt("help",   "",     "This help text.");
 		str += genUsageSection("Main options:", list);
 
@@ -635,6 +621,12 @@ private:
 		str +=
 		"# which/make detection adapted from Qt\n"
 		"which_command() {\n"
+		"	ALL_MATCHES=\n"
+		"	if [ \"$1\" = \"-a\" ]; then\n"
+		"		ALL_MATCHES=\"-a\"\n"
+		"		shift\n"
+		"	fi\n"
+		"\n"
 		"	OLD_HOME=$HOME\n"
 		"	HOME=/dev/null\n"
 		"	export HOME\n"
@@ -649,39 +641,35 @@ private:
 		"			for a in /usr/ucb /usr/bin /bin /usr/local/bin; do\n"
 		"				if [ -x $a/which ]; then\n"
 		"					WHICH=$a/which\n"
-		"					break;\n"
+		"					break\n"
 		"				fi\n"
 		"			done\n"
 		"		fi\n"
 		"	fi\n"
 		"\n"
+		"	RET_CODE=1\n"
 		"	if [ -z \"$WHICH\" ]; then\n"
 		"		OLD_IFS=$IFS\n"
 		"		IFS=:\n"
 		"		for a in $PATH; do\n"
 		"			if [ -x $a/$1 ]; then\n"
 		"				echo \"$a/$1\"\n"
-		"				IFS=$OLD_IFS\n"
-		"				export IFS\n"
-		"				HOME=$OLD_HOME\n"
-		"				export HOME\n"
-		"				return 0\n"
+		"				RET_CODE=0\n"
+		"				[ -z \"$ALL_MATCHES\" ] && break\n"
 		"			fi\n"
 		"		done\n"
 		"		IFS=$OLD_IFS\n"
 		"		export IFS\n"
 		"	else\n"
-		"		a=`\"$WHICH\" \"$1\" 2>/dev/null`\n"
+		"		a=`\"$WHICH\" \"$ALL_MATCHES\" \"$1\" 2>/dev/null`\n"
 		"		if [ ! -z \"$a\" -a -x \"$a\" ]; then\n"
 		"			echo \"$a\"\n"
-		"			HOME=$OLD_HOME\n"
-		"			export HOME\n"
-		"			return 0\n"
+		"			RET_CODE=0\n"
 		"		fi\n"
 		"	fi\n"
 		"	HOME=$OLD_HOME\n"
 		"	export HOME\n"
-		"	return 1\n"
+		"	return $RET_CODE\n"
 		"}\n"
 		"WHICH=which_command\n"
 		"\n";
@@ -754,8 +742,12 @@ private:
 		"			QC_VERBOSE=\"Y\"\n"
 		"			shift\n"
 		"			;;\n"
+		"		--qtselect=*)\n"
+		"			QC_QTSELECT=\"${optarg}\"\n"
+		"			shift\n"
+		"			;;\n"
 		"		--help) show_usage; exit ;;\n"
-		"		*) show_usage; exit ;;\n"
+		"		*) show_usage; exit 1;;\n"
 		"	esac\n"
 		"done\n\n";
 
@@ -771,117 +763,43 @@ private:
 		return str;
 	}
 
-	QString genQt3Checks()
-	{
-		QStringList qtloc;
-		qtloc += "/usr/lib/qt";
-		qtloc += "/usr/share/qt";
-		qtloc += "/usr/share/qt3";
-		qtloc += "/usr/local/lib/qt";
-		qtloc += "/usr/local/share/qt";
-		qtloc += "/usr/lib/qt3";
-		qtloc += "/usr/local/lib/qt3";
-		qtloc += "/usr/X11R6/share/qt";
-		qtloc += "/usr/qt/3";
-
-		QString for_items;
-		for(QStringList::ConstIterator it = qtloc.begin(); it != qtloc.end(); ++it) {
-			for_items += (*it) + ' ';
-		}
-
-		QString str =
-		"if [ -z \"$QTDIR\" ]; then\n"
-		"	if [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
-		"		echo \\$QTDIR not set... trying to find Qt manually\n"
-		"	fi\n";
-		str += QString(
-		"	for p in %1; do\n"
-		).arg(for_items);
-		str +=
-		"		if [ -d \"$p/mkspecs\" ]; then\n"
-		"			QTDIR=$p\n"
-		"			break;\n"
-		"		fi;\n"
-		"	done\n"
-		"	if [ -z \"$QTDIR\" ]; then\n"
-		"		echo fail\n"
-		"		echo\n"
-		"		echo \"Unable to find Qt 'mkspecs'.  Perhaps you need to\"\n"
-		"		echo \"install the Qt 3 development utilities.  You may download\"\n"
-		"		echo \"them either from the vendor of your operating system\"\n"
-		"		echo \"or from http://www.trolltech.com/\"\n"
-		"		echo\n"
-		"		echo \"If you're sure you have the Qt development utilities\"\n"
-		"		echo \"installed, you might try using the --qtdir option.\"\n"
-		"		echo\n"
-		"		exit 1;\n"
-		"	fi\n"
-		"	if [ ! -x \"$QTDIR/bin/moc\" ]; then\n"
-		"		m=`which moc 2>/dev/null`\n"
-		"		if [ ! -x \"$m\" ]; then\n"
-		"			echo fail\n"
-		"			echo\n"
-		"			echo \"We found Qt in $QTDIR, but we were unable to locate\"\n"
-		"			echo \"the moc utility.  It was not found in $QTDIR/bin\"\n"
-		"			echo \"nor in PATH.  This seems to be a very unusual setup.\"\n"
-		"			echo \"You might try using the --qtdir option.\"\n"
-		"			echo\n"
-		"			exit 1;\n"
-		"		fi\n"
-		"		qtpre=`echo $m | awk '{ n = index($0, \"/bin/moc\"); if (!n) { exit 1; } print substr($0, 0, n-1); exit 0; }' 2>/dev/null`\n"
-		"		ret=\"$?\"\n"
-		"		if [ \"$ret\" != \"0\" ]; then\n"
-		"			echo fail\n"
-		"			echo\n"
-		"			echo \"We found Qt in $QTDIR, but the location of moc\"\n"
-		"			echo \"($m) is not suitable for use with this build system.\"\n"
-		"			echo \"This is a VERY unusual and likely-broken setup.  You\"\n"
-		"			echo \"should contact the maintainer of your Qt package.\"\n"
-		"			echo\n"
-		"			exit 1;\n"
-		"		fi\n"
-		"		QTDIR=$qtpre\n"
-		"	fi\n"
-		"fi\n"
-		"\n"
-		"if [ ! -x \"$QTDIR/bin/qmake\" ]; then\n"
-		"	if [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
-		"		echo Warning: qmake not in \\$QTDIR/bin/qmake\n"
-		"		echo trying to find it in \\$PATH\n"
-		"	fi\n"
-		"	qm=`which qmake 2>/dev/null`\n"
-		"	if [ -x \"$qm\" ]; then\n"
-		"		if [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
-		"			echo qmake found in $qm\n"
-		"		fi\n"
-		"	else\n"
-		"		echo fail\n"
-		"		echo\n"
-		"		echo Sorry, you seem to have a very unusual setup,\n"
-		"		echo or I missdetected \\$QTDIR=$QTDIR\n"
-		"		echo\n"
-		"		echo Please set \\$QTDIR manually and make sure that\n"
-		"		echo \\$QTDIR/bin/qmake exists.\n"
-		"		echo\n"
-		"		exit 1;\n"
-		"	fi\n"
-		"else\n"
-		"	qm=$QTDIR/bin/qmake\n"
-		"fi\n\n";
-		return str;
-	}
-
 	QString genQt4Checks()
 	{
 		QString str =
-		"# run qmake -v and check version\n"
-		"qmake_check_v4() {\n"
+		"if [ -z \"$QC_QTSELECT\" ]; then\n"
+		"	QC_QTSELECT=\"$(echo $QT_SELECT | tr -d \"qt\")\"\n"
+		"fi\n"
+		"\n"
+		"if [ ! -z \"$QC_QTSELECT\" ]; then\n"
+		"	QTSEARCHTEXT=\"$QC_QTSELECT\"\n"
+		"else\n"
+		"	QTSEARCHTEXT=\"4 or 5\"\n"
+		"fi\n"
+		"\n"
+		"# run qmake and check version\n"
+		"qmake_check() {\n"
 		"	if [ -x \"$1\" ]; then\n"
-		"       local v=`$1 -query QT_VERSION 2>&1`\n"
-		"       vmaj=\"${v%%.*}\"\n"
-		"       case \"${v}\" in ?.?.?) [ \"$vmaj\" = 4 -o \"$vmaj\" = 5 ] && return 0 ;; esac\n"
+		"		cmd=\"\\\"$1\\\" -query QT_VERSION\"\n"
 		"		if [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
-		"			echo \"Warning: $1 not for Qt 4 or 5\"\n"
+		"			echo \"running: $cmd\"\n"
+		"		fi\n"
+		"		vout=`/bin/sh -c \"$cmd\" 2>&1`\n"
+		"		case \"${vout}\" in\n"
+		"			?.?.?)\n"
+		"				vmaj=\"${vout%%.*}\"\n"
+		"				if [ ! -z \"$QC_QTSELECT\" ]; then\n"
+		"					if [ \"$vmaj\" = \"$QC_QTSELECT\" ]; then\n"
+		"						return 0\n"
+		"					fi\n"
+		"				else\n"
+		"					if [ \"$vmaj\" = \"4\" ] || [ \"$vmaj\" = \"5\" ]; then\n"
+		"						return 0\n"
+		"					fi\n"
+		"				fi\n"
+		"				;;\n"
+		"		esac\n"
+		"		if [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
+		"			echo \"Warning: $1 not for Qt ${QTSEARCHTEXT}\"\n"
 		"		fi\n"
 		"	fi\n"
 		"	return 1\n"
@@ -894,65 +812,119 @@ private:
 		"fi\n"
 		"\n"
 		"qm=\"\"\n"
-		"names=\"qmake-qt5 qmake5 qmake-qt4 qmake4 qmake\"\n"
+		"qt4_names=\"qmake-qt4 qmake4\"\n"
+		"qt5_names=\"qmake-qt5 qmake5\"\n"
+		"names=\"qmake\"\n"
+		"if [ -z \"$QC_QTSELECT\" ]; then\n"
+		"	names=\"${qt5_names} ${qt4_names} $names\"\n"
+		"else\n"
+		"	if [ \"$QC_QTSELECT\" = \"4\" ]; then\n"
+		"		names=\"${qt4_names} $names\"\n"
+		"	elif [ \"$QC_QTSELECT\" -ge \"5\" ]; then\n"
+		"		names=\"${qt5_names} $names\"\n"
+		"	fi\n"
+		"fi\n"
 		"\n"
-		"# qt4 check: --qtdir\n"
 		"if [ -z \"$qm\" ] && [ ! -z \"$EX_QTDIR\" ]; then\n"
+		"	# qt4 check: --qtdir\n"
 		"	for n in $names; do\n"
 		"		qstr=$EX_QTDIR/bin/$n\n"
-		"		if qmake_check_v4 \"$qstr\"; then\n"
+		"		if qmake_check \"$qstr\"; then\n"
 		"			qm=$qstr\n"
-		"			break;\n"
+		"			break\n"
 		"		fi\n"
 		"	done\n"
-		"fi\n"
-		"if [ -z \"$qm\" ] && [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
-		"	echo \"Warning: qmake not found via --qtdir\"\n"
-		"fi\n"
+		"	if [ -z \"$qm\" ] && [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
+		"		echo \"Warning: qmake not found via --qtdir\"\n"
+		"	fi\n"
 		"\n"
-		"# qt4 check: QTDIR\n"
-		"if [ -z \"$qm\" ] && [ ! -z \"$QTDIR\" ]; then\n"
+		"elif [ -z \"$qm\" ] && [ ! -z \"$QTDIR\" ]; then\n"
+		"	# qt4 check: QTDIR\n"
 		"	for n in $names; do\n"
 		"		qstr=$QTDIR/bin/$n\n"
-		"		if qmake_check_v4 \"$qstr\"; then\n"
+		"		if qmake_check \"$qstr\"; then\n"
 		"			qm=$qstr\n"
-		"			break;\n"
+		"			break\n"
 		"		fi\n"
 		"	done\n"
-		"fi\n"
-		"if [ -z \"$qm\" ] && [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
-		"	echo \"Warning: qmake not found via \\$QTDIR\"\n"
-		"fi\n"
+		"	if [ -z \"$qm\" ] && [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
+		"		echo \"Warning: qmake not found via \\$QTDIR\"\n"
+		"	fi\n"
 		"\n"
-		"# qt4 check: pkg-config\n"
-		"if [ -z \"$qm\" ]; then\n"
-		"	str=`pkg-config QtCore --variable=exec_prefix 2>/dev/null`\n"
-		"	if [ ! -z \"$str\" ]; then\n"
+		"else\n"
+		"	# Try all other implicit checks\n"
+		"\n"
+		"	# qtchooser\n"
+		"	if [ -z \"$qm\" ]; then\n"
+		"		qtchooser=$($WHICH qtchooser 2>/dev/null)\n"
+		"		if [ ! -z \"$qtchooser\" ]; then\n"
+		"			if [ ! -z \"$QC_QTSELECT\" ]; then\n"
+		"				versions=\"$QC_QTSELECT\"\n"
+		"			else\n"
+		"				cmd=\"$qtchooser --list-versions\"\n"
+		"				if [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
+		"					echo \"running: $cmd\"\n"
+		"				fi\n"
+		"				versions=`$cmd`\n"
+		"			fi\n"
+		"			for version in $versions; do\n"
+		"				cmd=\"$qtchooser -run-tool=qmake -qt=${version} -query QT_INSTALL_BINS\"\n"
+		"				if [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
+		"					echo \"running: $cmd\"\n"
+		"				fi\n"
+		"				qtbins=`$cmd 2>/dev/null`\n"
+		"				if [ ! -z \"$qtbins\" ] && qmake_check \"$qtbins/qmake\"; then\n"
+		"					qm=\"$qtbins/qmake\"\n"
+		"					break\n"
+		"				fi\n"
+		"			done\n"
+		"		fi\n"
+		"	fi\n"
+		"	if [ -z \"$qm\" ] && [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
+		"		echo \"Warning: qmake not found via qtchooser\"\n"
+		"	fi\n"
+		"\n"
+		"	# qt4: pkg-config\n"
+		"	if [ -z \"$qm\" ]; then\n"
+		"		cmd=\"pkg-config QtCore --variable=exec_prefix\"\n"
+		"		if [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
+		"			echo \"running: $cmd\"\n"
+		"		fi\n"
+		"		str=`$cmd 2>/dev/null`\n"
+		"		if [ ! -z \"$str\" ]; then\n"
+		"			for n in $names; do\n"
+		"				qstr=$str/bin/$n\n"
+		"				if qmake_check \"$qstr\"; then\n"
+		"					qm=$qstr\n"
+		"					break\n"
+		"				fi\n"
+		"			done\n"
+		"		fi\n"
+		"	fi\n"
+		"	if [ -z \"$qm\" ] && [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
+		"		echo \"Warning: qmake not found via pkg-config\"\n"
+		"	fi\n"
+		"\n"
+		"	# qmake in PATH\n"
+		"	if [ -z \"$qm\" ]; then\n"
 		"		for n in $names; do\n"
-		"			qstr=$str/bin/$n\n"
-		"			if qmake_check_v4 \"$qstr\"; then\n"
-		"				qm=$qstr\n"
-		"				break;\n"
+		"			qstr=`$WHICH -a $n 2>/dev/null`\n"
+		"			for q in $qstr; do\n"
+		"				if qmake_check \"$q\"; then\n"
+		"					qm=\"$q\"\n"
+		"					break\n"
+		"				fi\n"
+		"			done\n"
+		"			if [ ! -z \"$qm\" ]; then\n"
+		"				break\n"
 		"			fi\n"
 		"		done\n"
 		"	fi\n"
-		"fi\n"
-		"if [ -z \"$qm\" ] && [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
-		"	echo \"Warning: qmake not found via pkg-config\"\n"
-		"fi\n"
+		"	if [ -z \"$qm\" ] && [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
+		"		echo \"Warning: qmake not found via \\$PATH\"\n"
+		"	fi\n"
 		"\n"
-		"# qt4 check: PATH\n"
-		"if [ -z \"$qm\" ]; then\n"
-		"	for n in $names; do\n"
-		"		qstr=`$WHICH $n 2>/dev/null`\n"
-		"		if qmake_check_v4 \"$qstr\"; then\n"
-		"			qm=$qstr\n"
-		"			break;\n"
-		"		fi\n"
-		"	done\n"
-		"fi\n"
-		"if [ -z \"$qm\" ] && [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
-		"	echo \"Warning: qmake not found via \\$PATH\"\n"
+		"	# end of implicit checks\n"
 		"fi\n"
 		"\n"
 		"if [ -z \"$qm\" ]; then\n"
@@ -964,7 +936,7 @@ private:
 
 		str += echoBlock(1,
 		"\n"
-		"Reason: Unable to find the 'qmake' tool for Qt 4.\n"
+		"Reason: Unable to find the 'qmake' tool for Qt ${QTSEARCHTEXT}.\n"
 		"\n");
 		str +=
 		"	show_qt_info\n";
@@ -972,7 +944,7 @@ private:
 		"	exit 1;\n"
 		"fi\n"
 		"if [ \"$QC_VERBOSE\" = \"Y\" ]; then\n"
-		"	echo qmake found in $qm\n"
+		"	echo qmake found in \"$qm\"\n"
 		"fi\n\n";
 
 		str +=
@@ -983,7 +955,7 @@ private:
 		"		READLINK=`$WHICH readlink`\n"
 		"	fi\n"
 		"	if [ ! -z \"$READLINK\" ]; then\n"
-		"		qt_mkspecsdir=`$qm -query QT_INSTALL_DATA`/mkspecs\n"
+		"		qt_mkspecsdir=`\"$qm\" -query QT_INSTALL_DATA`/mkspecs\n"
 		"		if [ -d \"$qt_mkspecsdir\" ] && [ -h \"$qt_mkspecsdir/default\" ]; then\n"
 		"			defmakespec=`$READLINK $qt_mkspecsdir/default`\n"
 		"		fi\n"
@@ -1011,34 +983,28 @@ private:
 	QString genDoQConf()
 	{
 		QString outdir = ".qconftemp";
-		QString cleanup = QString("rm -rf %1").arg(outdir);
+		QString cleanup = QString("rm -rf \"%1\"").arg(outdir);
 
 		QString str;
 		str += QString("%1\n").arg(cleanup);
 
 		str += QString(
 		"(\n"
-		"	mkdir %1\n"
-		"	gen_files %2\n"
-		"	cd %3\n"
+		"	mkdir \"%1\"\n"
+		"	gen_files \"%2\"\n"
+		"	cd \"%3\"\n"
 		).arg(outdir).arg(outdir).arg(outdir);
 
 		if(qt4) {
 			str +=
 			"	if [ ! -z \"$qm_spec\" ]; then\n"
-			"		$qm -spec $qm_spec conf4.pro >/dev/null\n"
+			"		\"$qm\" -spec $qm_spec conf4.pro >/dev/null\n"
 			"	else\n"
-			"		$qm conf4.pro >/dev/null\n"
+			"		\"$qm\" conf4.pro >/dev/null\n"
 			"	fi\n"
 			"	$MAKE clean >/dev/null 2>&1\n"
 			"	$MAKE >../conf.log 2>&1\n";
 		}
-		else {
-			str +=
-			"	$qm conf.pro >/dev/null\n"
-			"	QTDIR=$QTDIR make clean >/dev/null 2>&1\n"
-			"	QTDIR=$QTDIR make >../conf.log 2>&1\n";
-		}
 		str += ")\n\n";
 
 		if(qt4) {
@@ -1071,33 +1037,13 @@ private:
 			"	exit 1;\n"
 			"fi\n\n";
 		}
-		else {
-			str += QString(
-			"if [ \"$?\" != \"0\" ]; then\n"
-			"	%1\n"
-			"	echo fail\n"
-			"	echo\n"
-			"	echo \"There was an error compiling 'conf'.  Be sure you have a proper\"\n"
-			"	echo \"Qt 3.x Multithreaded (MT) build environment set up.  This\"\n"
-			"	echo \"means not just Qt, but also a C++ compiler, the 'make' command,\"\n"
-			"	echo \"and any other packages necessary to compile C++ programs.\"\n"
-			"	echo \"See conf.log for details.\"\n"
-			"	if [ ! -f \"$QTDIR/lib/libqt-mt.so.3\" ]; then\n"
-			"		echo\n"
-			"		echo \"One possible reason is that you don't have\"\n"
-			"		echo \"libqt-mt.so.3 installed in $QTDIR/lib/.\"\n"
-			"	fi\n"
-			"	echo\n"
-			"	exit 1;\n"
-			"fi\n\n").arg(cleanup);
-		}
 
 		if(qt4) {
 			str += QString("QC_COMMAND=$0\n");
 			str += QString("export QC_COMMAND\n");
 			str += QString("QC_PROFILE=%1\n").arg(profile);
 			str += QString("export QC_PROFILE\n");
-			str += QString("QC_QMAKE=$qm\n");
+			str += QString("QC_QMAKE=\"$qm\"\n");
 			str += QString("export QC_QMAKE\n");
 			str += QString("QC_QMAKESPEC=$qm_spec\n");
 			str += QString("export QC_QMAKESPEC\n");
@@ -1105,7 +1051,7 @@ private:
 			str += QString("export QC_MAKETOOL\n");
 		}
 
-		str += QString("%1/conf\n").arg(outdir);
+		str += QString("\"%1/conf\"\n").arg(outdir);
 
 		str += "ret=\"$?\"\n";
 		str += "if [ \"$ret\" = \"1\" ]; then\n";
@@ -1124,14 +1070,10 @@ private:
 			"			echo \"fail\"\n"
 			"		fi\n";
 		}
-		else
-			str += "		echo fail\n";
 
 		str += "		echo\n";
 		if(qt4)
 			str += "		echo \"Reason: Unexpected error launching 'conf'\"\n";
-		else
-			str += "		echo \"Unexpected error launching 'conf'\"\n";
 		str += "		echo\n";
 		str += "		exit 1;\n";
 		str += "	fi\n";
@@ -1154,7 +1096,7 @@ private:
 	{
 		QString str;
 		str += "# run qmake\n";
-		str += QString("$qm %1\n").arg(profile);
+		str += QString("\"$qm\" %1\n").arg(profile);
 		str +=
 		"if [ \"$?\" != \"0\" ]; then\n"
 		"	echo\n"
@@ -1181,7 +1123,7 @@ private:
 	QString genEmbeddedFile(const QString &name, const QByteArray &a)
 	{
 		QString str;
-		str += QString("cat >%1 <<EOT\n").arg(name);
+		str += QString("cat >\"%1\" <<EOT\n").arg(name);
 		str += escapeFile(QString::fromLatin1(a));
 		str += "\nEOT\n";
 		return str;
@@ -1552,8 +1494,15 @@ int main(int argc, char **argv)
 
 	QDir confdir(confdirpath);
 	if(!confdir.exists()) {
-		printf("qconf: %s does not exist.\n", qPrintable(confdir.path()));
-		return 1;
+#ifdef Q_OS_WIN
+		confdir = QDir(appdir + "/../share/qconf/conf");
+		if(!confdir.exists()) {
+#endif
+			printf("qconf: %s does not exist.\n", qPrintable(confdir.absolutePath()));
+			return 1;
+#ifdef Q_OS_WIN
+		}
+#endif
 	}
 
 	QFile f;
@@ -1584,6 +1533,7 @@ int main(int argc, char **argv)
 	QByteArray confpro = f.readAll();
 	f.close();
 	confpro += "\nDEFINES += HAVE_MODULES\n";
+	confpro += "\ngreaterThan(QT_MAJOR_VERSION, 4):CONFIG += c++11\n";
 
 	printf("Project name: %s\n", qPrintable(conf.name));
 	printf("Profile: %s\n", qPrintable(conf.profile));
@@ -1660,19 +1610,7 @@ int main(int argc, char **argv)
 		dep.section = info.section;
 		dep.args = info.args;
 
-		// prepend #line
-		int oldsize = buf.size();
-		QString str = QString("#line 1 \"%1\"\n").arg(modfname);
-		QByteArray cs = str.toLocal8Bit();
-		int len = cs.length();
-		buf.resize(oldsize + len);
-		memmove(buf.data() + len, buf.data(), oldsize);
-		memcpy(buf.data(), cs.data(), len);
-
-		// append to the module buffer
-		oldsize = allmods.size();
-		allmods.resize(oldsize + buf.size());
-		memcpy(allmods.data() + oldsize, buf.data(), buf.size());
+		allmods += (QString("#line 1 \"%1\"\n").arg(modfname).toLocal8Bit() + buf);
 
 		modscreate += QString("    o = new qc_%1(conf);\n    o->required = %2;\n    o->disabled = %3;\n"
 			).arg(escapeArg(dep.name)).arg(dep.required ? "true": "false").arg(dep.disabled ? "true": "false");
